#!/usr/bin/perl -w
# 
# sb::Admin::Category - Module for Serene Bach
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb::Admin::Category;

use strict;

# ==================================================
# // module version
# ==================================================
use vars qw( $VERSION @ISA );
$VERSION = '0.00';
# 0.00 [2005/02/02] generated

# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Config ();
use sb::Language ();
use sb::Interface ();
use sb::TemplateManager ();
use sb::Data ();
use sb::Text ();
use sb::Admin::List ();
@ISA = qw( sb::Admin::List );
# ==================================================
# // declaration for constant value
# ==================================================
sub TEMPLATE       (){ 'category.html' };
sub LIST_COLUMN    (){ 8 };
sub DEFAULT_COLUMN (){ '-' };
sub ORDER_LEFT     (){ '<input type="submit" name="up%d" value="&#9650;" class="updown" />' };
sub ORDER_RIGHT    (){ '<input type="submit" name="dn%d" value="&#9660;" class="updown" />' };
sub ORDER_COLUMN   (){ '%s</td><td>%s' };
sub NAVI_ARROW     (){ ' &gt; ' };
# ==================================================
# // declaration for class member
# ==================================================
# ==================================================
# // public functions - callback
# ==================================================
sub callback { # Хå
	my $self = shift;
	return ($self->{'regi'}) 
		? $self->_update_category(@_) 
		: $self->_display_category_list(@_);
}
# ==================================================
# // private functions - main routine
# ==================================================
sub _update_category {
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi = sb::Interface->get;
	my $lang = sb::Language->get;
	my $msg  = '';
	my $main = ( $cgi->value('cid') ne '' ) ? sb::Data->load('Category','id'=>$cgi->value('cid')) : undef;
	$self->{'cat'} = { sb::Data->load_as_hash('Category') };
	if ($cgi->value('__regi') eq 'add') { # ɲ
		if ($cgi->value('cat_name') ne '') { # ƥ꡼̾ɬ
			$main = $self->create_category(
				'name' => sb::Text->entitize($cgi->value('cat_name')),  # ƥ꡼̾
				'main' => $cgi->value('cat_main'),                      # ƥƥ꡼
				'sub'  => ($cgi->value('cat_main') ne '') ? 1 : undef,  # ҥƥ꡼Ȥ뤫ɤ
				'text' => sb::Text->entitize($cgi->value('cat_text')),  # 
			);
			$msg = $lang->string('parts_new_comp');
		} else {
			return $self->process_message($lang->string('error_no_cat'));
		}
	} elsif ($cgi->value('__regi') eq 'update') { # ǡ
		my @cats = sb::Data->load('Category', # оݥƥ꡼ɤ߽Ф
			'cond'  => {'id'=>[ split("\0",$cgi->value('sel_id')) ]},
			'order' => 1,
			'sort'  => 'order'
		);
		if ( $cgi->value('del') ne '' ) { # 
			my @selected = split("\0",$cgi->value('sel'));
			my @dels = ();
			foreach (@cats) {
				my $id = $_->id;
				if ( grep(/^\Q$id\E$/,@selected) ) {
					push(@dels,$_);
					push(@dels,$self->_get_subs($_)) if ($_->sub ne '');
				}
			}
			foreach (@dels) {
				$_->erase;
			}
			sb::Data->update(@dels) if (@dels);
			$msg = ($#dels + 1) . $lang->string('parts_deleted');
		} else { # ̾ѹ or ¤ؤ
			my $order = undef;
			foreach (@cats) {
				my $name = $cgi->value('cat_name' . $_->id);
				$_->name(sb::Text->entitize($name)) if ($name ne '' and $cgi->value('update') ne '');
				$order = $_ if ($cgi->value('dn' . $_->id) ne '' or $cgi->value('up' . $_->id) ne '');
			}
			if ($order) { # ¤ؤ
				@cats = $self->change_order(
					'data'      => \@cats,
					'target'    => $order,
					'direction' => ($cgi->value('up' . $order->id) ne '') ? +1 : -1,
				);
			}
			$msg = $lang->string('parts_editcomp');
		}
		sb::Data->update(@cats) if (@cats and $cgi->value('del') eq '');
		if ($main) { # ֥ƥ꡼ѹ
			my @subs = ();
			foreach (sort { $b->order <=> $a->order } @cats) {
				next if ($_->erased);
				push(@subs,$_->id);
			}
			$main->sub(join(',',@subs) . ',');
			sb::Data->update($main);
		}
	} elsif ( $cgi->value('cid') ne '' ) { # ܺ٥ǡ
		return $self->process_message($lang->string('error_no_cat')) if (!$main); # ƥ꡼ʤ
		if ( $cgi->value('detail') ne '' ) { # ƥ꡼
			$main->name(sb::Text->entitize($cgi->value('cat_name'))) if ($cgi->value('cat_name') ne '');
			$main->text(sb::Text->entitize($cgi->value('cat_text')));
			$main->url($cgi->value('cat_tb'));
			$main->dir($cgi->value('cat_dir')) if ($cgi->value('cat_dir') ne '');
			$main->temp($cgi->value('cat_temp'));
			$main->idx($cgi->value('cat_index'));
			$main->disp($cgi->value('cat_disptop') . ':' . $cgi->value('cat_displist') . ':');
			if ($cgi->value('cat_main') ne $main->main) { # ƥƥ꡼ѹ
				my $flag = undef;
				my $old = ($main->main ne '') 
				        ? $self->{'cat'}->{$main->main} 
				        : undef;
				my $new = ($cgi->value('cat_main') ne '') 
				        ? $self->{'cat'}->{$cgi->value('cat_main')} 
				        : undef;
				foreach my $chk_cat ( values(%{$self->{'cat'}}) ) {
					next if ($chk_cat->name ne $main->name);
					$flag = 1 if ($new and $chk_cat->main eq $cgi->value('cat_main')); # Ʊҥƥ꡼
					$flag = 1 if (!$new and $chk_cat->main eq ''); # Ʊƥƥ꡼
					last if ($flag);
				}
				if (!$flag) { # ̾ʣƤʤäưǽ
					$main->main($cgi->value('cat_main'));
					if ($new) { # ҥƥ꡼ȤϿƥƥ꡼⹹
						$new->add_sub($main->id);
						sb::Data->update($new);
					}
					if ($old) {
						$old->remove_sub($main->id);
						sb::Data->update($old);
					}
				} else {
					$msg = $lang->string('error_dup_cat');
				}
			}
		} elsif ( $cgi->value('del') ne '' ) { # ƥ꡼
			my $parent = ($main->main ne '') ? $self->{'cat'}->{$main->main} : undef;
			my @dels = ( $main );
			push(@dels,$self->_get_subs($main)) if ($main->sub ne '');
			foreach (@dels) {
				$_->erase;
			}
			if ($parent) {
				$parent->remove_sub($main->id);
				sb::Data->update($parent);
			}
			sb::Data->update(@dels);
			return $self->process_message(($#dels + 1) . $lang->string('parts_deleted'));
		}
		if ($main) {
			sb::Data->update($main);
			$msg .= $lang->string('parts_editcomp');
		}
	} else {
		die($lang->string('error_unknown') . "\n");
	}
	$self->build_list('category_list');
	return $self->_display_category_list('message'=>$msg,'id'=>($main) ? $main->id : undef);
}
sub _display_category_list {
	my $self = shift;
	my %param = (
		'message' => '',
		'id'      => undef,
		@_
	);
	my $cgi = sb::Interface->get;
	my $cms = sb::TemplateManager->new($self->load_template('file'=>TEMPLATE));
	my @cats = ();
	$self->{'cat'} = { sb::Data->load_as_hash('Category') };
	$self->{'cnt'} = $self->_count_entry;
	if ($cgi->value('__type') ne 'tree') { # Խ⡼
		$param{'id'} = $cgi->value('cid') if ($cgi->value('cid') ne '');
		if ($param{'id'} ne '') {
			if ($self->{'cat'}->{$param{'id'}}) {
				foreach my $cat ( split(',',$self->{'cat'}->{$param{'id'}}->sub) ) {
					next if ($cat eq '');
					next if (!$self->{'cat'}->{$cat});
					push(@cats,$self->{'cat'}->{$cat});
				}
			} else {
				return $self->process_message(sb::Language->get->string('error_no_cat'));
			}
		} else {
			foreach my $cat ( sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ) {
				next if ($cat->main ne '');
				push(@cats,$cat);
			}
		}
		$cms->num(0);
		$self->listmain(
			'template' => $cms,
			'block'    => 'sb_category_list',
			'objects'  => \@cats,
			'tags'     => {
				'sb_catlist_id'   => 'id',
				'sb_catlist_name' => \&_display_name,
				'sb_catlist_num'  => sub { int($self->{'cnt'}->{$_[1]->id}) },
				'sb_catlist_sub'  => \&_display_subcat_num,
				'sb_catlist_del'  => \&_display_checkbox,
				'sb_site_cgi'     => sub { $self->get_script_path },
			},
		);
		for (my $i=0;$i<@cats;$i++) { # 
			my $lcol = sprintf(ORDER_LEFT ,$cats[$i]->id);
			my $rcol = sprintf(ORDER_RIGHT,$cats[$i]->id);
			$lcol = DEFAULT_COLUMN if ($i == 0);
			$rcol = DEFAULT_COLUMN if ($i == $#cats);
			$cms->num($i);
			$cms->tag('sb_catlist_order'=>sprintf(ORDER_COLUMN,$lcol,$rcol));
		}
		if ( $param{'id'} ne '' and $self->{'cat'}->{$param{'id'}} ) {
			my $main = $self->{'cat'}->{$param{'id'}};
			$cms->num(0);
			$cms->tag('sb_cat_id'=>$main->id);
			$cms->tag('sb_cat_name'=>$main->name);
			$cms->tag('sb_cat_text'=>$main->text);
			$cms->tag('sb_cat_tb'=>$main->url);
			foreach my $key ('top','list') {
				$self->select_option(
					'cms'      => $cms,
					'tag'      => 'sb_cat_disp' . $key . '_',
					'selected' => $main->get_option($key),
				);
			}
			$self->template_selector(
				'cms' => $cms,
				'tag' => 'sb_cat_temp',
				'now' => ($main->temp == -1) ? undef : $main->temp,
			);
			if (sb::Config->get->value('conf_entry_archive') ne 'None') { # ¸
				my @dirs = sort { $a cmp $b } sb::Config->get->writable_dir(sb::Config->get->value('conf_dir_log'));
				my $selector = '';
				foreach my $dir (@dirs) { # ǥ쥯ȥ
					$selector .= '<option value="' . $dir . '"';
					$selector .= ' selected="selected"' if ($main->dir eq $dir);
					$selector .= '>' . $dir . '</option>';
				}
				$cms->num(0);
				$cms->tag('sb_cat_dir'=>$selector);
				$cms->tag('sb_cat_index_' . $main->idx=>'selected="selected"') if ($main->idx);
				$cms->block('sb_category_dir'=>1);
				$cms->block('sb_category_index'=>1);
			}
			my $navi = $self->_category_navigator(
				'id'  => $main->id,
				'tag' => '<a href="' . $self->get_script_path . '?__mode=category&amp;cid=%d">%s</a>',
				'now' => $main->id,
			);
			$navi .= ' (<a href="' . $self->get_script_path . '?__mode=list&amp;dispcat=';
			$navi .= $main->id . '">' . int($self->{'cnt'}->{$main->id}) . '</a>)';
			$cms->num(0);
			$cms->tag('sb_cat_navi'=>$navi);
			$cms->tag('sb_cat_selector'=>
				$self->category_selector(
					'cat'    => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
					'now'    => $main->main,
					'except' => $main->id,
				)
			);
		} else {
			$cms->num(0);
			$cms->tag('sb_cat_selector'=>
				$self->category_selector('cat' => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ])
			);
		}
		$cms->num(0);
		$cms->tag('sb_submenu_category_basic'=>' class="current"');
		$cms->block('sb_category_select'=>($param{'id'} eq '') ? 1 : 0);
		$cms->block('sb_category_one'   =>($param{'id'} eq '') ? 0 : 1);
		$cms->block('sb_category_basic' =>1);
		$cms->block('sb_category_tree'  =>0);
	} else { # ĥ꡼⡼
		$cms->num(0);
		$cms->tag('sb_category_tree'=>
			$self->_category_tree(
				'cat'  => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
				'num'  => $self->{'cnt'},
				'path' => $self->get_script_path . '?__mode=category&amp;cid=',
			)
		);
		$cms->tag('sb_submenu_category_tree'=>' class="current"');
		$cms->block('sb_category_select'=>0);
		$cms->block('sb_category_one'   =>0);
		$cms->block('sb_category_basic' =>0);
		$cms->block('sb_category_tree'  =>1);
	}
	if ($param{'message'} ne '') { # 
		$cms->num(0);
		$cms->tag('sb_process_message'=>$param{'message'});
		$cms->block('sb_category_message'=>1);
	}
	$self->common_template_parts($cms);
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}
# ==================================================
# // private functions - for category list
# ==================================================
sub _get_subs { # ֥ƥ꡼Ƶ
	my $self = shift;
	my $cat  = shift;
	my @subs = split(',',$cat->sub);
	my @objs = ();
	foreach (@subs) {
		my $obj = $self->{'cat'}->{$_};
		if ($obj) {
			push(@objs,$obj) ;
			push(@objs,$self->_get_subs($obj)) if ($obj->sub ne '');
		}
	}
	return @objs;
}
sub _category_tree { # ƥ꡼ĥ꡼
	my $self  = shift;
	my %param = (
		'cat'    => [],
		'branch' => undef,   
		'num'    => {},
		'path'   => undef,
		'class'  => -1,
		@_
	);
	my $list = '';
	my $num = $param{'class'};
	foreach my $cat ( @{$param{'cat'}} ) {
		next if (!defined($param{'branch'}) and $cat->main ne '');
		next if ( defined($param{'branch'}) and $cat->main ne $param{'branch'});
		my $dir = (sb::Config->get->value('conf_entry_archive') ne 'Individual') 
			? undef
			: ($cat->dir ne '') ? $cat->dir : sb::Config->get->value('conf_dir_log');
		$num++;
		$list .= ($num % 2) ? '<li class="odd">' : '<li class="even">';
		$list .= '<span class="dir">' . $dir . '</span>' if ($dir);
		$list .= '<a href="' . $param{'path'} . $cat->id . '">' . $cat->name . '</a>';
		$list .= ' (' . int($param{'num'}->{$cat->id}) . ')';
		if ($cat->sub ne '') {
			$list .= "\n" . '<ul>';
			$list .= $self->_category_tree(
				'cat'    => $param{'cat'},
				'branch' => $cat->id,
				'num'    => $param{'num'},
				'path'   => $param{'path'},
				'class'  => $num,
			);
			$list .= '</ul>';
		}
		$list .= '</li>' . "\n";
	}
	return($list);
}
sub _category_navigator {
	my $self = shift;
	my %param = (
		'id'  => undef,
		'tag' => '',
		'now' => undef,
		@_
	);
	return( undef ) if (!defined($param{'id'}));
	return( undef ) if (!$self->{'cat'}->{$param{'id'}});
	if ( $self->{'cat'}->{$param{'id'}}->main eq '') {
		return ($param{'now'} eq $param{'id'})
			? $self->{'cat'}->{$param{'id'}}->name
			: sprintf($param{'tag'},$param{'id'},$self->{'cat'}->{$param{'id'}}->name);
	} else {
		my $navi = $self->_category_navigator(
			'id'  => $self->{'cat'}->{$param{'id'}}->main,
			'tag' => $param{'tag'},
			'now' => $param{'now'},
		);
		$navi .= NAVI_ARROW if ($navi);
		return ($param{'now'} eq $param{'id'})
			? $navi . $self->{'cat'}->{$param{'id'}}->name
			: $navi . sprintf($param{'tag'},$param{'id'},$self->{'cat'}->{$param{'id'}}->name);
	}
}
sub _display_name {
	my $self = shift;
	my $obj  = shift;
	return '<input type="text" name="cat_name' . $obj->id . '" value="' . $obj->name . '" size="24" style="width:240px;" class="text" />';
}
sub _display_subcat_num {
	my $self = shift;
	my $obj  = shift;
	my $sub  = $obj->sub;
	return int( $sub =~ s/,/,/g );
}
sub _display_checkbox {
	my $self = shift;
	my $obj  = shift;
	return '<input type="checkbox" name="sel" value="' . $obj->id . '" onclick="switchList(this)" />';
}
sub _count_entry {
	my $self = shift;
	my %count = ();
	my @array = sb::Data->load('Entry');
	for (my $i=0;$i<@array;$i++) {
		next if ($array[$i]->cat eq '');
		my $cat = $self->{'cat'}->{$array[$i]->cat};
		next if (!$cat);
		$count{$cat->id}++;
		if ($array[$i]->add ne '') { # Ϣƥ꡼Υ
			foreach ( split(',',$array[$i]->add) ) {
				next if ($_ eq '');
				$count{$_}++;
			}
		}
	}
	return \%count;
}
1;
__END__
